<?php

namespace App\Service;

use App\Entity\Answer;
use App\Entity\Question;
use App\Repository\AnswerRepository;
use App\Repository\QuestionRepository;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Class QuestionService
 * @package App\Service
 */
class QuestionService
{
    private QuestionRepository $questionRepository;

    private AnswerRepository $answerRepository;

    public function __construct(QuestionRepository $questionRepository, AnswerRepository $answerRepository)
    {
        $this->questionRepository = $questionRepository;
        $this->answerRepository = $answerRepository;
    }

    public function generateNextQuestion(string $topic, array $excludeIds = []) : array
    {
        /** @var Question[] $questions */
        $questions = $this->questionRepository->findAllByTopicExcludeIds($topic, $excludeIds);
        if (count($questions) === 0) {
            throw new NotFoundHttpException('Next question not found!');
        }

        /** @var Question $target */
        $target = $questions[array_rand($questions)];

        if ($target->isSingleChoice() || $target->isMultipleChoiceRandomWrongs()) {
            /** @var Answer[] $additionalAnswers */
            $additionalAnswers = $this->answerRepository->findAllByTopicAndContentType($topic, $target->getContentType());
            if (count($additionalAnswers) === 0) {
                throw new NotFoundHttpException('Additional answers not found');
            }

            for ($i = 0; $i < (5 - $target->getAnswers()->count()); $i++) {
                /** @var int $rnd */
                $rnd = array_rand($additionalAnswers);
                $target->addAnswer($additionalAnswers[$rnd]);
                unset($additionalAnswers[$rnd]);
            }
        }

        return [
            'id' => $target->getId(),
            'type' => $target->getStructureType(),
            'label' => $target->getLabel(),
            'answers' => array_map(function ($answer) {
                /** @var Answer $answer */
                return [
                    'id' => $answer->getId(),
                    'label' => $answer->getLabel()
                ];
            }, $target->getAnswers()->getValues())
        ];
    }

    public function evaluateQuestion(int $questionId, array $answers) : bool
    {
        return false;
    }
}
